"use strict";

// Sub-entry of a multi-entry form entry
// (e.g. one phone number among several phone numbers grouped in a single entry)
class SubEntry {
    constructor(subEntryElement, index, parentFormEntry) {
        this.parentFormEntry = parentFormEntry;
        this.index = index;

        this.subEntryElement = subEntryElement;
        this.removeButtonElement = subEntryElement.querySelector(".remove-button");
        this.deletionCheckboxElement = subEntryElement.querySelector("input[id$='DELETE']");

        this.addPlaceholderAttributes();
        this.startHandlingSubEntryRemoveButtonClicks();
    }

    addPlaceholderAttributes() {
        // Add the type placeholder to the first input element
        // and the value placeholder to the second one
        const inputElements = this.subEntryElement.querySelectorAll("input");
        inputElements[0].setAttribute("placeholder", this.parentFormEntry.placeholders.type);
        inputElements[1].setAttribute("placeholder", this.parentFormEntry.placeholders.value);
    }

    hide() {
        this.subEntryElement.classList.add("hidden");
    }

    display() {
        this.subEntryElement.classList.remove("hidden");
    }

    markForDeletionAndHide() {
        // Check Django's deletion checkbox
        this.deletionCheckboxElement.checked = true;

        this.hide();
    }

    reset() {
        // Clear all input fields
        for (let inputElement of this.subEntryElement.querySelectorAll("input")) {
            inputElement.value = "";
        }

        // Uncheck Django's deletion checkbox
        this.deletionCheckboxElement.checked = false;
    }

    resetAndDisplay() {
        this.reset();
        this.display();
    }

    reindex(newIndex) {
        // Replace -<old index>- by -<new index>- in the values of the
        // id and name attributes of every child of this sub-entry's element
        const attributesToUpdate = ["id", "name"];

        for (let childElement of this.subEntryElement.childNodes) {
            for (let attribute of attributesToUpdate) {
                if (!childElement.hasAttribute(attribute)) {
                    continue;
                }

                const newValue = childElement
                    .getAttribute(attribute)
                    .replace(`-${this.index}-`, `-${newIndex}-`);
                childElement.setAttribute(attribute, newValue);
            }
        }

        this.index = newIndex;
    }

    startHandlingSubEntryRemoveButtonClicks() {
        this.removeButtonElement.addEventListener("click", event => {
            event.preventDefault();
            this.parentFormEntry.removeSubEntry(this);
        });
    }

    static fromTemplate(templateElement, index, parentFormEntry) {
        // Clone the template, update its class,
        // and replace __prefix__ by the index of the new sub-entry
        const clonedTemplateElement = templateElement.cloneNode(true);
        clonedTemplateElement.innerHTML = clonedTemplateElement.innerHTML
            .replace(/__prefix__/g, index.toString());
        clonedTemplateElement.className = "form-sub-entry";

        return new SubEntry(clonedTemplateElement, index, parentFormEntry);
    }
}



// ----------------------------------------------------------------------------



// Form entry which can contain 0+ sub-entries
// (e.g. a phone number entry which can contain several numbers)
class MultiEntryFormEntry {
    constructor(formEntryElement) {
        this.formEntryElement = formEntryElement;
        this.subEntryTemplateElement = formEntryElement.querySelector(".form-sub-entry-template");
        this.newSubEntryButtonElement = formEntryElement.querySelector(".add-sub-entry-button");
        this.totalFormsElement = [...formEntryElement.querySelectorAll("input")]
            .find(element => element.name && element.name.endsWith("TOTAL_FORMS"));
        this.maxNumFormsElement = [...formEntryElement.querySelectorAll("input")]
            .find(element => element.name && element.name.endsWith("MAX_NUM_FORMS"));

        this.placeholders = {
            type: this.formEntryElement.getAttribute("data-type-placeholder"),
            value: this.formEntryElement.getAttribute("data-value-placeholder")
        };

        this.maxNbSubEntries = parseInt(this.maxNumFormsElement.value);
        this.initialNbSubEntries = 0;
        this.subEntries = [];
        this.createInitialSubEntries();

        this.startHandlingNewSubEntryButtonClicks();
    }

    get nbSubEntries() {
        return this.subEntries
            .filter(subEntry =>
                !subEntry.subEntryElement.classList.contains("hidden")
            )
            .length;
    }

    get hasHiddenSubEntries() {
        return this.subEntries
            .findIndex(subEntry =>
                subEntry.subEntryElement.classList.contains("hidden")
            ) >= 0;        
    }

    createInitialSubEntries() {
        const subEntryElements = this.formEntryElement.querySelectorAll(".form-sub-entry");
        this.initialNbSubEntries = subEntryElements.length;

        // This loop assumes sub-entries elements are
        // ordered with no gap according to indices which start at 0
        for (let [index, element] of [...subEntryElements].entries()) {
            this.subEntries.push(
                new SubEntry(element, index, this)
            );
        }
    }

    createNewSubEntry() {
        if (this.nbSubEntries === this.maxNbSubEntries) {
            console.log(`The max. number of sub-entries has been reached (${this.maxNbSubEntries}).`);
            return;
        }

        // If there are hidden sub-entries,
        // it means one of the initial sub-entry elements should be reset and displayed
        if (this.hasHiddenSubEntries) {
            // Reset and display the first hidden sub-entry
            const existingSubEntry = this.subEntries.find(
                subEntry => subEntry.subEntryElement.classList.contains("hidden")
            );

            existingSubEntry.resetAndDisplay();
        }

        // Otherwise, it means a new sub-entry (element) must be created
        // and appended to the form entry element
        else {
        const newSubEntryIndex = this.nbSubEntries;
        const newSubEntry = SubEntry.fromTemplate(
                this.subEntryTemplateElement,
                newSubEntryIndex,
                this
            );
            
            this.subEntries.push(newSubEntry);
            this.formEntryElement.insertBefore(
                newSubEntry.subEntryElement,
                this.newSubEntryButtonElement
            );

            // Increment Django's TOTAL_FORMS hidden form input
            this.totalFormsElement.value = (parseInt(this.totalFormsElement.value) + 1).toString();
        }
    }

    removeSubEntry(subEntry) {
        // If the index of the sub-entry to remove is below the initial number of sub-entries,
        // it means one of the initial sub entry elements should be marked for deletion and hidden
        const removedSubEntryIndex = subEntry.index;
        if (removedSubEntryIndex < this.initialNbSubEntries) {
            subEntry.markForDeletionAndHide();
        }

        // Otherwise, delete the sub-entry (and remove its element from the DOM)
        // and reindex other user-created sub-entries if need be
        else {
            subEntry.subEntryElement.remove();

            this.subEntries.splice(subEntry.index, 1);
            for (let index = this.removeSubEntryIndex; index < this.subEntries.length; index++) {
                this.subEntries[index].reindex(index);
            }

            // Decrement Django's TOTAL_FORMS hidden form input
            this.totalFormsElement.value = (parseInt(this.totalFormsElement.value) - 1).toString();
        }
    }

    startHandlingNewSubEntryButtonClicks() {
        const buttonElement = this.formEntryElement.querySelector(".add-sub-entry-button");
        buttonElement.addEventListener("click", event => {
            event.preventDefault();
            this.createNewSubEntry();
        });
    }
}



// ----------------------------------------------------------------------------



// Setup the script by creating one instance of MultiEntryFormEntry
// for each form entry configured to contain several sub-entries
document.addEventListener("DOMContentLoaded", event => {
    const multiEntryFormEntryElements = document.querySelectorAll(
        "#content-edit-profile .form-entry.multi-entry"
    );

    for (let element of multiEntryFormEntryElements) {
        const formEntry = new MultiEntryFormEntry(element);
    }
});

